package com.simuwang.manage.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.read.listener.PageReadListener;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.alibaba.excel.util.StringUtils;
import com.simuwang.base.common.conts.ExcelConst;
import com.simuwang.base.common.enums.ContactStatusType;
import com.simuwang.base.common.enums.SignMethodType;
import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.common.util.DateUtils;
import com.simuwang.base.common.util.StringUtil;
import com.simuwang.base.mapper.daq.CompanyContactInfoMapper;
import com.simuwang.base.mapper.daq.ContactInformationMapper;
import com.simuwang.base.mapper.daq.LastCompanyContactInfoMapper;
import com.simuwang.base.pojo.dto.CompanyContactExcelData;
import com.simuwang.base.pojo.dos.*;
import com.simuwang.base.pojo.dto.query.CompanyContactQuery;
import com.simuwang.base.pojo.dto.query.CompanyIdQuery;
import com.simuwang.base.pojo.vo.*;
import com.simuwang.manage.service.CompanyContactService;
import com.simuwang.manage.service.CompanyInformationService;
import com.simuwang.shiro.utils.UserUtils;
import com.smppw.common.pojo.ResultVo;
import com.smppw.common.pojo.enums.status.ResultCode;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
public class CompanyContactServiceImpl implements CompanyContactService {

    @Autowired
    private LastCompanyContactInfoMapper lastCompanyContactInfoMapper;

    @Autowired
    private CompanyContactInfoMapper companyContactInfoMapper;

    @Autowired
    private ContactInformationMapper contactInformationMapper;

    @Autowired
    private CompanyInformationService companyInformationService;

    @Value("${email.file.path}")
    private String path;

    private static final Logger logger = LoggerFactory.getLogger(CompanyContactServiceImpl.class);
    @Override
    public ResultVo saveCompanyContactInfo(CompanyContactInfoVO companyContactInfoVO) {
        ResultVo vo = new ResultVo(ResultCode.SAVE_SUCCESS.getCode());
        companyContactInfoVO.setUpdateTime(DateUtils.getTime());
        companyContactInfoVO.setIsvalid(1);
        companyContactInfoVO.setUpdaterId(UserUtils.getLoginUser().getUserId());
        CompanyContactInfoDO companyContactInfoDO = BeanUtil.copyProperties(companyContactInfoVO,CompanyContactInfoDO.class);
        CompanyContactInfoDO oldCompanyContactInfoDO = companyContactInfoMapper.getCompanyContactInfo(companyContactInfoDO.getCompanyId(),DateUtils.format(companyContactInfoDO.getContactDate(), DateUtils.YYYY_MM_DD));
        if(oldCompanyContactInfoDO != null && companyContactInfoVO.getId() == null){
            vo.setData(false);
            vo.setMsg("对接信息已经存在，无需新增");
            return vo;
        }
        if(oldCompanyContactInfoDO == null){
            companyContactInfoVO.setCreateTime(DateUtils.getTime());
            companyContactInfoVO.setCreatorId(UserUtils.getLoginUser().getUserId());
            companyContactInfoMapper.insert(companyContactInfoDO);
        }else{
            companyContactInfoDO.setId(oldCompanyContactInfoDO.getId());
            companyContactInfoMapper.updateById(companyContactInfoDO);
        }
        LastCompanyContactInfoDO lastCompanyContactInfoDO = lastCompanyContactInfoMapper.getLastCompanyContactInfoDOByCompanyId(companyContactInfoVO.getCompanyId());
        if (lastCompanyContactInfoDO == null) {
            lastCompanyContactInfoDO = BeanUtil.copyProperties(companyContactInfoDO,LastCompanyContactInfoDO.class);
            lastCompanyContactInfoMapper.insert(lastCompanyContactInfoDO);
        }else{
            String contactDate = companyContactInfoVO.getContactDate();
            if(Objects.requireNonNull(DateUtils.parse(contactDate, DateUtils.YYYY_MM_DD)).compareTo(lastCompanyContactInfoDO.getContactDate()) >= 0){
                companyContactInfoDO.toLastDO(lastCompanyContactInfoDO);
                lastCompanyContactInfoMapper.updateById(lastCompanyContactInfoDO);
            }
        }
        vo.setData(true);
        return vo;
    }

    @Override
    public MybatisPage<CompanyContactInfoPageVO> selectCompanyContactInfoList(CompanyContactQuery companyContactQuery) {
        List<CompanyContactInfoPageDO> contactInformationDOList = companyContactInfoMapper.selectCompanyContactInfoList(companyContactQuery);
        long total = companyContactInfoMapper.countCompanyContactInfo(companyContactQuery);
        List<CompanyContactInfoPageVO> dataList = contactInformationDOList.stream().map(CompanyContactInfoPageDO::toVO).collect(Collectors.toList());
        return MybatisPage.of(total,dataList);
    }

    @Override
    public MybatisPage<CompanyContactPageVO> selectCompanyContactList(CompanyIdQuery companyIdQuery) {
        List<CompanyContactPageDO> contactInformationDOList = companyContactInfoMapper.selectCompanyContactListByCompanyId(companyIdQuery);
        long total = companyContactInfoMapper.countCompanyContactByCompanyId(companyIdQuery);
        List<CompanyContactPageVO> dataList = contactInformationDOList.stream().map(CompanyContactPageDO::toVO).collect(Collectors.toList());
        return MybatisPage.of(total,dataList);
    }

    @Override
    public ResultVo selectCompanyContactDetail(IdVO idVO) {
        CompanyContactPageDO pageDO = companyContactInfoMapper.selectCompanyContactDetail(idVO.getId());
        return new ResultVo(pageDO);
    }

    @Override
    public void deleteById(Integer id) {
        CompanyContactInfoDO companyContactInfoDO = companyContactInfoMapper.selectById(id);
        Integer userId = UserUtils.getLoginUser().getUserId();
        LastCompanyContactInfoDO lastCompanyContactInfoDO = lastCompanyContactInfoMapper.getLastCompanyContactInfoDOByCompanyId(companyContactInfoDO.getCompanyId());
        companyContactInfoMapper.deleteCompanyContactInfoById(id,userId);
        if(lastCompanyContactInfoDO.getContactDate().equals(companyContactInfoDO.getContactDate())){
            //说明删除的是最新的对接信息，此时需要重新信息最新的对接信息
            CompanyContactInfoDO nextCompanyContactInfoDO = companyContactInfoMapper.getLastCompanyContactInfoDO(companyContactInfoDO.getCompanyId());
            if(nextCompanyContactInfoDO == null){
                //说明没有对接人信息了，直接删除最新数据表的记录
                lastCompanyContactInfoMapper.deleteLastById(lastCompanyContactInfoDO.getId(),userId);
            }else{
                //更新最新数据
                lastCompanyContactInfoDO = BeanUtil.copyProperties(nextCompanyContactInfoDO,LastCompanyContactInfoDO.class);
                lastCompanyContactInfoDO.setId(null);
                lastCompanyContactInfoDO.setUpdateTime(new Date());
                lastCompanyContactInfoDO.setCreateTime(new Date());
                lastCompanyContactInfoDO.setIsvalid(1);
                lastCompanyContactInfoDO.setUpdaterId(userId);
                lastCompanyContactInfoDO.setCreatorId(userId);
                lastCompanyContactInfoMapper.insert(lastCompanyContactInfoDO);
            }
        }
    }

    @Override
    public ResultVo uploadCompanyContact(MultipartFile excelFile) {
        ResultVo vo = new ResultVo(ResultCode.SUCCESS);
        File file = null;
        try{
            InputStream inputStream = excelFile.getInputStream();
            file = new File(path+"/upload/"+ System.currentTimeMillis()+"/"+excelFile.getOriginalFilename());
            FileUtils.copyInputStreamToFile(inputStream,file);
            List<CompanyContactExcelData> list = parseContactFile(file);
            vo.setData(parseResult(list));
        }catch (Exception e){
            logger.error(e.getMessage(),e);
            vo.setMsg("文件解析异常");
            vo.setData(false);
            return vo;
        }
        return vo;
    }

    private List<CompanyContactExcelData> parseContactFile(File file) {
        // 创建一个 list 存储每行的数据，即 ExcelData 对象
        List<CompanyContactExcelData> list = new ArrayList<>();
        // 直接使用 EasyExcel 的 read 方法，同时定义表头的类型，以便将列中数据映射为 ExcelData 对象
        EasyExcel.read(file, CompanyContactExcelData.class, new PageReadListener<CompanyContactExcelData>(dataList -> {
            // 并且每行数据，并将其 add 至 list 中
            for (CompanyContactExcelData excelData : dataList) {
                if (excelData != null) {
                    list.add(excelData);
                }
            }
        })).excelType(ExcelTypeEnum.XLSX).sheet().doRead();
        return list;
    }
    private Object parseResult(List<CompanyContactExcelData> list) {
        CompanyContactUploadResult result = new CompanyContactUploadResult();
        int startRow = 1;
        List<CompanyContactExcelSuccessDataVO> successDataVOList = new ArrayList<>();
        List<CompanyContactExcelFailDataVO> excelFailDataVOList = new ArrayList<>();
        for(int dataIdx=0;dataIdx<list.size(); dataIdx++){
            CompanyContactExcelData excelData = list.get(dataIdx);
            try{
                if((StringUtil.isEmpty(excelData.getContactName()) || StringUtil.isEmpty(excelData.getContactDate()) || StringUtil.isEmpty(excelData.getCompanyId()))
                        || StringUtil.isEmpty(excelData.getContactStatus())  || StringUtil.isEmpty(excelData.getContactEmail()) || (StringUtil.isEmpty(excelData.getContactPhone()) && StringUtil.isEmpty(excelData.getContactWechat()))
                ){
                    CompanyContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData, ExcelConst.REQUIRE_FIELD,dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                    continue;
                }
                String companyName= companyInformationService.getCompanyNameById(excelData.getCompanyId());
                if(StringUtil.isEmpty(companyName)){
                    CompanyContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData, ExcelConst.ERROR_COMPANY_ID,dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                    continue;
                }
                Date contactDate = null;
                try{
                    contactDate = DateUtils.parse(excelData.getContactDate(),DateUtils.YYYY_MM_DD);
                }catch (Exception e){
                    CompanyContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData, ExcelConst.ERROR_DATE_FORMAT,dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                    continue;
                }
                if(ContactStatusType.getContactStatusTypeByInfo(excelData.getContactStatus()) == null ||
                        (StringUtil.isNotEmpty(excelData.getSignMethod()) && SignMethodType.getSignMethodTypeByInfo(excelData.getSignMethod()) == null)){
                    CompanyContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData, ExcelConst.ERROR_METHOD,dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                    continue;
                }

                if(ContactStatusType.getContactStatusTypeByInfo(excelData.getContactStatus()) == null ||
                        (StringUtil.isNotEmpty(excelData.getSignMethod()) && SignMethodType.getSignMethodTypeByInfo(excelData.getSignMethod()) == null)){
                    CompanyContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData, ExcelConst.ERROR_METHOD,dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                    continue;
                }
                //开始处理成功数据
                ContactInformationDO contactInformationDO = new ContactInformationDO();
                contactInformationDO.setContactName(excelData.getContactName());
                contactInformationDO.setContactEmail(excelData.getContactEmail());
                contactInformationDO.setContactWechat(excelData.getContactWechat());
                contactInformationDO.setContactPhone(excelData.getContactPhone());
                contactInformationDO.setCompanyId(excelData.getCompanyId());
                boolean successFlag = saveContactInfo(contactInformationDO);
                if(!successFlag){
                    CompanyContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData,"对接信息数据入库异常",dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                    continue;
                }
                CompanyContactInfoDO companyContactInfDO = new CompanyContactInfoDO();
                companyContactInfDO.setContactor(excelData.getContactor());
                companyContactInfDO.setContactId(contactInformationDO.getId());
                companyContactInfDO.setContactDate(contactDate);
                companyContactInfDO.setCustomerManager(excelData.getCustomerManager());
                companyContactInfDO.setContactRemark(excelData.getContactRemark());
                companyContactInfDO.setCompanyId(excelData.getCompanyId());
                companyContactInfDO.setContactStatus(ContactStatusType.getContactStatusTypeByInfo(excelData.getContactStatus()).getCode());
                companyContactInfDO.setSignMethod(StringUtils.isEmpty(excelData.getSignMethod())?null:SignMethodType.getSignMethodTypeByInfo(excelData.getSignMethod()).getCode());
                companyContactInfDO.setCreatorId(UserUtils.getLoginUser().getUserId());
                companyContactInfDO.setUpdaterId(UserUtils.getLoginUser().getUserId());
                companyContactInfDO.setUpdateTime(new Date());
                companyContactInfDO.setCreateTime(new Date());
                companyContactInfDO.setIsvalid(1);
                boolean saveFlag = saveCompanyContact(companyContactInfDO);
                if(saveFlag){
                    CompanyContactExcelSuccessDataVO successDataVO = new CompanyContactExcelSuccessDataVO();
                    successDataVO.setContactor(excelData.getContactor());
                    successDataVO.setContactDate(excelData.getContactDate());
                    successDataVO.setCustomerManager(excelData.getCustomerManager());
                    successDataVO.setCompanyId(excelData.getCompanyId());
                    successDataVO.setContactName(excelData.getContactName());
                    successDataVO.setContactStatus(excelData.getContactStatus());
                    successDataVO.setSignMethod(excelData.getSignMethod());
                    successDataVO.setContactRemark(excelData.getContactRemark());
                    successDataVO.setContactEmail(excelData.getContactEmail());
                    successDataVO.setContactPhone(excelData.getContactPhone());
                    successDataVO.setContactWechat(excelData.getContactWechat());
                    successDataVO.setCompanyName(companyName);
                    successDataVOList.add(successDataVO);
                }else{
                    CompanyContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData,"对接信息数据入库异常",dataIdx+startRow);
                    excelFailDataVOList.add(failDataVO);
                }
            }catch (Exception e){
                logger.error(e.getMessage(),e);
                CompanyContactExcelFailDataVO failDataVO = toExcelFailDataVO(excelData,e.getMessage(),dataIdx+startRow);
                excelFailDataVOList.add(failDataVO);
            }
        }
        result.setSuccess(successDataVOList);
        result.setFail(excelFailDataVOList);
        result.setTotal(successDataVOList.size()+excelFailDataVOList.size());
        result.setFailCount(excelFailDataVOList.size());
        result.setSuccessCount(successDataVOList.size());
        return result;
    }

    private boolean saveContactInfo(ContactInformationDO contactInformationDO) {
        try{
            ContactInformationDO contactInformation = contactInformationMapper.selectByKey(contactInformationDO);
            Integer userId = UserUtils.getLoginUser().getUserId();
            if(contactInformation != null){
                contactInformationDO.setId(contactInformation.getId());
                contactInformationDO.setIsvalid(1);
                contactInformationDO.setUpdateTime(new Date());
                contactInformationDO.setUpdaterId(userId);
                contactInformationDO.setContactRemark(contactInformation.getContactRemark());
                contactInformationDO.setFileId(contactInformation.getFileId());
                contactInformationMapper.updateById(contactInformationDO);
            }else{
                contactInformationDO.setIsvalid(1);
                contactInformationDO.setUpdateTime(new Date());
                contactInformationDO.setUpdaterId(userId);
                contactInformationDO.setCreateTime(new Date());
                contactInformationDO.setCreatorId(userId);
                contactInformationMapper.insert(contactInformationDO);
            }
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            return false;
        }
        return  true;
    }

    private boolean saveCompanyContact(CompanyContactInfoDO companyContactInfDO) {
        try{
            CompanyContactInfoDO oldCompanyContactInfoDO = companyContactInfoMapper.getCompanyContactInfo(companyContactInfDO.getCompanyId(),DateUtils.format(companyContactInfDO.getContactDate(), DateUtils.YYYY_MM_DD));
            if(oldCompanyContactInfoDO == null){
                companyContactInfDO.setCreateTime(new Date());
                companyContactInfDO.setCreatorId(UserUtils.getLoginUser().getUserId());
                companyContactInfoMapper.insert(companyContactInfDO);
            }else{
                companyContactInfDO.setId(oldCompanyContactInfoDO.getId());
                companyContactInfDO.setUpdateTime(new Date());
                companyContactInfDO.setCreateTime(oldCompanyContactInfoDO.getCreateTime());
                companyContactInfDO.setCreatorId(oldCompanyContactInfoDO.getCreatorId());
                companyContactInfDO.setUpdaterId(UserUtils.getLoginUser().getUserId());
                companyContactInfoMapper.updateById(companyContactInfDO);
            }
            LastCompanyContactInfoDO lastCompanyContactInfoDO = lastCompanyContactInfoMapper.getLastCompanyContactInfoDOByCompanyId(companyContactInfDO.getCompanyId());
            if (lastCompanyContactInfoDO == null) {
                lastCompanyContactInfoDO = BeanUtil.copyProperties(companyContactInfDO,LastCompanyContactInfoDO.class);
                lastCompanyContactInfoMapper.insert(lastCompanyContactInfoDO);
            }else{
                Date contactDate = companyContactInfDO.getContactDate();
                if(contactDate.compareTo(lastCompanyContactInfoDO.getContactDate()) >= 0){
                    companyContactInfDO.toLastDO(lastCompanyContactInfoDO);
                    lastCompanyContactInfoMapper.updateById(lastCompanyContactInfoDO);
                }
            }
        }catch (Exception e){
            logger.error(e.getMessage(),e);
            return false;
        }
        return true;
    }

    private CompanyContactExcelFailDataVO toExcelFailDataVO(CompanyContactExcelData excelData, String msg,Integer rowNum) {
        CompanyContactExcelFailDataVO failDataVO = new CompanyContactExcelFailDataVO();
        failDataVO.setContactor(excelData.getContactor());
        failDataVO.setContactDate(excelData.getContactDate());
        failDataVO.setCustomerManager(excelData.getCustomerManager());
        failDataVO.setContactName(excelData.getContactName());
        failDataVO.setCompanyId(excelData.getCompanyId());
        failDataVO.setContactStatus(excelData.getContactStatus());
        failDataVO.setSignMethod(excelData.getSignMethod());
        failDataVO.setContactRemark(excelData.getContactRemark());
        failDataVO.setContactEmail(excelData.getContactEmail());
        failDataVO.setContactPhone(excelData.getContactPhone());
        failDataVO.setContactWechat(excelData.getContactWechat());
        failDataVO.setCompanyName(companyInformationService.getCompanyNameById(excelData.getCompanyId()));
        failDataVO.setFailReason(msg);
        failDataVO.setRowNum(rowNum);
        return failDataVO;
    }
}
